// This javascript is executed only on chrome://settings and http://127.0.0.1:8080 (JMUConfig). The code here can be used to customize
// chrome settings, add, hide and modify entries


function getSettingsSection(sectName) {

 var mainPage = document.getElementsByTagName("settings-ui")[0].shadowRoot.querySelector("#main").shadowRoot;
 return mainPage.querySelector("settings-basic-page").shadowRoot.querySelector("#" + sectName);
}

function handleToggle(id, setting_name) {
   var container = getSettingsSection(id);
   if (container) {
       var toggle = container.shadowRoot.querySelector("cr-toggle");
       if (toggle) {
          if (settings[setting_name])
             toggle.setAttribute("checked", true);
          else
             toggle.removeAttribute("checked");

          container.onpointerup = function(e) {
             var settings = {};
             settings[setting_name] = !toggle.hasAttribute("checked");
             chrome.extension.sendMessage("", { cmd: "setSettings", settings: settings });
             if ( toggle.hasAttribute("checked") )
                toggle.removeAttribute("checked");
             else
                toggle.setAttribute("checked", true);
             e.stopPropagation();
          };
       }
   }
}

var settings = {};
var flags = {};

document.addEventListener("DOMContentLoaded", function(e){

   chrome.runtime.sendMessage("", { cmd: "getInfo" }, function(info) {
      if ( info ) {
         if ( info.settings )
            settings = info.settings;

         if ( info.flags )
            flags = info.flags;
      }

      if ( document.location.href.startsWith('http://127.0.0.1:8080') ) {

         if (info.tabIndex != 1)
             return;

         var jmuconfigExit = null;

         // Hook to logout() action. We need a waiting loop as elements are
         // dynamically generated by angualar
         var timer = window.setInterval(function(){
            // This should match JMUConfig elements calling logout()
            jmuconfigExit = document.querySelectorAll('[ng-click="logout()"]');
            if (jmuconfigExit && jmuconfigExit.length > 0 ){
               window.clearInterval(timer);
               jmuconfigExit.forEach(function(exitAction) {
                  exitAction.addEventListener("click", function() {
                     chrome.extension.sendMessage("", { cmd: "back" });
                  });
               });
            }
         }, 1000);

         return;
      }


      // Tool Bar
      var enableToolBar = getSettingsSection("ext_enableToolBar");
      var enableToolBarToggle = enableToolBar.shadowRoot.querySelector("cr-toggle");
      var enableToolBarSettings = getSettingsSection("toolBarSettings");

      if (enableToolBar && enableToolBarToggle) {
         if (settings.enableNavBar) {
            enableToolBarToggle.setAttribute("checked", true);
            if (enableToolBarSettings)
               enableToolBarSettings.style="display: block"
         } else {
            enableToolBarToggle.removeAttribute("checked");
            if (enableToolBarSettings)
               enableToolBarSettings.style="display: none"
         }

         enableToolBar.onpointerup = function(e) {
            chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'enableNavBar': !enableToolBarToggle.hasAttribute("checked") } });
            if ( enableToolBarToggle.hasAttribute("checked") ) {
               enableToolBarToggle.removeAttribute("checked");
               if (enableToolBarSettings)
                  enableToolBarSettings.style="display: none"
            } else {
               enableToolBarToggle.setAttribute("checked", true);
               if (enableToolBarSettings)
                  enableToolBarSettings.style="display: block"
            }
            e.stopPropagation();
         };
      }

      handleToggle("ext_autoNavBar", "autoNavBar");
      handleToggle("ext_historyBtns", "historyBtns");
      handleToggle("ext_loadingBar", "loadingBar");
      handleToggle("ext_wifiStatus", "wifiStatus");

      var enableCustomUserAgent = getSettingsSection("ext_customUserAgent");
      var customUserAgentC = getSettingsSection("ext_customUserAgentC");
      var enableCustomUserAgentString = getSettingsSection("ext_customUserAgentString");
      var changeUserAgent = getSettingsSection("ext_changeUserAgent");

      if (enableCustomUserAgent &&
          enableCustomUserAgentString &&
          changeUserAgent && customUserAgentC) {
         var enableCustomUserAgentToggle = enableCustomUserAgent.shadowRoot.querySelector("cr-toggle");
         var enableCustomUserAgentStringInput = enableCustomUserAgentString.shadowRoot.querySelector("input");

         if (enableCustomUserAgentToggle &&
             enableCustomUserAgentStringInput) {
            if (settings.customUserAgent !== null) {
               enableCustomUserAgentToggle.setAttribute("checked", true);
               enableCustomUserAgentStringInput.value = settings.customUserAgent;
               customUserAgentC.style="display: block";
            } else {
               enableCustomUserAgentToggle.removeAttribute("checked");
               customUserAgentC.style="display: none";
               enableCustomUserAgentStringInput.value = navigator.userAgent;
            }

            enableCustomUserAgent.onpointerup = function(e) {
               if ( enableCustomUserAgent.hasAttribute("checked") ) {
                  enableCustomUserAgent.removeAttribute("checked");
                  customUserAgentC.style="display: none";
                  enableCustomUserAgentStringInput.value = navigator.userAgent;
                  chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'customUserAgent': null } });
               } else {
                  enableCustomUserAgent.setAttribute("checked", true);
                  customUserAgentC.style="display: block";
                  enableCustomUserAgentStringInput.value = navigator.userAgent;
               }
               e.stopPropagation();
            };

            changeUserAgent.onclick = function(e) {
               chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'customUserAgent': enableCustomUserAgentStringInput.value } });
               e.stopPropagation();
            };
         }
      }

      var holdTimer = getSettingsSection("ext_holdTimer");
      if (holdTimer) {
         var holdTimerInput = holdTimer.shadowRoot.querySelector("input");
         var changeHoldTimer = getSettingsSection("ext_changeHoldTimer");
         if (holdTimerInput && changeHoldTimer) {
            if (settings.holdOptionsTimerMilliseconds)
               holdTimerInput.value = settings.holdOptionsTimerMilliseconds / 1000;
            else
               holdTimerInput.value = "3.5";
            changeHoldTimer.addEventListener("click", function(e) {
               var ht = holdTimerInput.value;
               ht = ht * 1000;
               if (ht)
                  chrome.extension.sendMessage("", { cmd: "setSettings",
                                                     settings: { 'holdOptionsTimerMilliseconds': ht } });
               else
                  holdTimerInput.value = settings.holdOptionsTimerMilliseconds / 1000;
               e.stopPropagation();
            });
         }
      }

      var fallbackPage = getSettingsSection("ext_fallbackPage");
      if (fallbackPage) {
         var fallbackPageInput = fallbackPage.shadowRoot.querySelector("input");
         var changeFallbackPage = getSettingsSection("ext_changeFallbackPage");
         if (fallbackPageInput && changeFallbackPage) {
            if (settings.fallbackPage)
               fallbackPageInput.value = settings.fallbackPage;
            else
               fallbackPageInput.value = "";
            changeFallbackPage.addEventListener("click", function(e) {
               var fp = fallbackPageInput.value;
               if (!fp)
                  fp = null;
               chrome.extension.sendMessage("", { cmd: "setSettings",
                                                  settings: { 'fallbackPage': fp } });
               e.stopPropagation();
            });
         }
      }

      // OnStartup radiobox
      if ( ! flags.homepage ) {
          getSettingsSection("onStartup").style = "display: block";
          var onStartupSection = getSettingsSection("onStartup").querySelector("settings-on-startup-page").shadowRoot;

          var onStartupSettings = onStartupSection.querySelector("#ext_startupOpenSettings");
          var onStartupContinue = onStartupSection.querySelector("#ext_startupContinue");
          var onStartupHomepage = onStartupSection.querySelector("#ext_startupOpenHomepage");

          if ( onStartupSettings && onStartupContinue && onStartupHomepage ) {

             if ( !settings.onStartup || settings.onStartup == "settings" )
                onStartupSettings.setAttribute("checked", true);
             else if ( settings.onStartup == "lastVisitedPage" )
                onStartupContinue.setAttribute("checked", true);
             else if ( settings.onStartup == "homepage" )
                onStartupHomepage.setAttribute("checked", true);

             onStartupSettings.onpointerup = function() {
                onStartupContinue.removeAttribute("checked");
                onStartupHomepage.removeAttribute("checked");
                chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'onStartup': "settings" } });
             };

             onStartupContinue.onpointerup = function() {
                chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'onStartup': "lastVisitedPage" } });
                onStartupSettings.removeAttribute("checked");
                onStartupHomepage.removeAttribute("checked");
             };

             onStartupHomepage.onpointerup = function() {
                chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'onStartup': "homepage" } });
                onStartupSettings.removeAttribute("checked");
                onStartupContinue.removeAttribute("checked");
             };
         }
      }

      var onStartupUrlSection = onStartupSection.querySelector("settings-startup-urls-page").shadowRoot;

      var homepageLabel = onStartupUrlSection.querySelector("#ext_homepage");
      if (homepageLabel && settings.homepage)
         homepageLabel.innerHTML = settings.homepage; 

      var homepageChange = onStartupUrlSection.querySelector("#ext_changeHomepage");
      var homepageText = onStartupUrlSection.querySelector("paper-input").shadowRoot.querySelector("paper-input-container").querySelector("input");
      if (homepageText && homepageChange) {
         homepageChange.onclick = function(e) {
            e.preventDefault();
            var url = homepageText.value;

            if ( ! url )
               return
            if ( url.indexOf("://") == -1 )
               url = "http://" + url;

            chrome.extension.sendMessage("", { cmd: "setSettings", settings: { 'homepage': url } });
            if (homepageLabel)
               homepageLabel.innerHTML = url;
         };
      }
   });

   // "Back" button on the left settings menu
   var backEntry = document.getElementsByTagName("settings-ui")[0].shadowRoot.querySelector("cr-toolbar").shadowRoot.querySelector("#ext_back");
   if (backEntry) {
      backEntry.addEventListener("pointerup", function() {
         chrome.extension.sendMessage("", { cmd: "back" });
      });
   }

   // "Panel Settings" button on the left settings menu
   var jmuconfigEntry = document.getElementsByTagName("settings-ui")[0].shadowRoot.querySelector("cr-toolbar").shadowRoot.querySelector("#ext_systemSettings");
   if (jmuconfigEntry) {
      jmuconfigEntry.addEventListener("pointerup", function() {
         chrome.extension.sendMessage("", { cmd: "jmuconfig" });
      });
   }

   // "Reload pages" button on the left settings menu
   var reloadPages = document.getElementById("ext_clearCache");
   if (reloadPages) {
      reloadPages.addEventListener("pointerup", function() {
         chrome.extension.sendMessage("", { cmd: "clearCache" });
      });
   }
});

// Disable context menu
window.addEventListener("contextmenu", function(e) { e.preventDefault(); })
