jlog('Setting up Chromium Controller');

app.controller('jmuconfig.chromium.Controller', 
[ '$scope', '$rootScope', '$window', '$http', '$interval', '$filter', 'ngDialog', 'defines', 'utils', 'auth', 'poller',
function ($scope, $rootScope, $window, $http, $interval, $filter, ngDialog, defines, utils, auth, poller)
{
  jlog('Initialising Chromium Controller');

  $rootScope.loading = true;
  $scope.formState = { open: false };

  utils.init();

  var chromiumUrl = defines.restApiUrl + '/app_chromium';

  var window = angular.element($window);

  $scope.showAlert = function(msg, warn, cb)
  {
    $rootScope.alert = { message: msg, warning: warn };

    $scope.dialog = ngDialog.openConfirm({
      template: 'alertDialog',
      className: 'ngdialog-theme-default',
      showClose: false
    }).then(function (value) {
    }, function (reason) {
       if (cb)
          cb();
    });
  };

  $scope.closeDialog = function()
  {
      if (!$scope.dialog)
          return;
      ngDialog.close($scope.dialog.id);
      $scope.dialog = null;
  };

  function pollPostCb()
  {
    if ( $scope.formState.open || ! $rootScope.global.apps || ! $rootScope.global.apps.chromium || ! $rootScope.global.apps.chromium.settings )
        return;

    $scope.chromium = $rootScope.global.apps.chromium.settings;

    if(!$scope.chromium.customUserAgent)
        $scope.chromium.customUserAgent = "";
    $scope.chromium.enableCustomUserAgent = $scope.chromium.customUserAgent != "";

    $scope.chromium.enableFallbackPage = $scope.chromium.fallbackPage && $scope.chromium.fallbackPage != "";
    $scope.chromium.holdOptionsTimer = $scope.chromium.holdOptionsTimerMilliseconds/1000;

    if ( $scope.chromium.prefs && $scope.chromium.prefs.services ) {
       $scope.chromium.allowDownload = $scope.chromium.prefs.services.downloadRestrictions != 3;
    }

    $scope.tmpChromium = angular.copy($scope.chromium);
  };

  $scope.saveData = function($event) {

    $rootScope.loading = true;

    if (!$scope.tmpChromium.enableCustomUserAgent)
        $scope.tmpChromium.customUserAgent = "";
    var userAgentWarn = $scope.tmpChromium.customUserAgent != $scope.chromium.customUserAgent;

    if (!$scope.tmpChromium.enableFallbackPage)
        $scope.tmpChromium.fallbackPage = "";

    if ( $scope.tmpChromium.prefs && $scope.tmpChromium.prefs.services ) {
        $scope.tmpChromium.prefs.services.downloadRestrictions = $scope.tmpChromium.allowDownload ? 0 : 3;
    }

    $scope.chromium = angular.copy($scope.tmpChromium);

    // make sure stale data is not polled in the meantime
    poller.skip();

    $http.post(chromiumUrl, $scope.tmpChromium)
        .success(function(data, status, headers, config) {
          $rootScope.loading = false;
          $scope.showAlert($filter('translate')('_SUCCESS'), userAgentWarn ? $filter('translate')('_USER_AGENT_WARN') : null);
        })
        .error(function(data, status, headers, config) {
            if (data && data.error) {
                $scope.showAlert($filter('translate')('_CHROMIUM_SAVE_ERROR') + ': ' + data.error);
                $scope.cancelChromiumForm();
            }
            else {
                $scope.showAlert($filter('translate')('_JMU_SERVICE_ERROR'));
            }
            poller.unskip();
            $rootScope.loading = false;
        });
  };

  $scope.cancelChromiumForm = function($event) {

    jlog('Cancelled Chromium Setting ');
    $scope.tmpChromium = angular.copy($scope.chromium);
  };

  $scope.$on('$destroy', function (event) {

    jlog('Destroying Chromium Controller');

    utils.deinit();
    poller.deinit();
  });

  poller.regPostCb(pollPostCb);
  poller.subscribe(['chromium']);
  pollPostCb();

  $rootScope.viewTitle = $filter('translate')('_BROWSER');

  $rootScope.loading = false;
}]);
